MarketplaceJobPostingsSearchQuery = """query marketplaceJobPostingsSearch($marketPlaceJobFilter: MarketplaceJobPostingsSearchFilter, $searchType: MarketplaceJobPostingSearchType, $sortAttributes: [MarketplaceJobPostingSearchSortAttribute]) {
  marketplaceJobPostingsSearch(
    marketPlaceJobFilter: $marketPlaceJobFilter
    searchType: $searchType
    sortAttributes: $sortAttributes
  ) {
    totalCount
    pageInfo {
      hasNextPage
      endCursor
    }
    edges {
      node {
        id
        title
        description
        category
        totalApplicants
        preferredFreelancerLocation
        preferredFreelancerLocationMandatory
        premium
        experienceLevel
        amount {
          rawValue
          currency
          displayValue
        }
        skills {
          name
          prettyName
          highlighted
        }
        hourlyBudgetType
        hourlyBudgetMin {
          rawValue
          currency
          displayValue
        }
        hourlyBudgetMax {
          rawValue
          currency
          displayValue
        }
        duration
        clientNotSureFields
        clientPrivateFields
        freelancerClientRelation{
          companyName
        }
        localJobUserDistance
        weeklyBudget {
          rawValue
          currency
          displayValue
        }
        engagementDuration {
          id
          label
          weeks
        }
        client{
          edcUserId
          totalHires
          totalPostedJobs
          totalSpent{
            rawValue
            currency
            displayValue
          }
          location{
            city
            country
            timezone
            state
            offsetToUTC
          }
          totalReviews
          totalFeedback
          companyRid
          companyName
          edcUserId
          lastContractPlatform
          lastContractRid
          lastContractTitle
          hasFinancialPrivacy
        }
        occupations{
          category{
            id
            prefLabel
          }
        }
        freelancerClientRelation{
          companyName
        }
        
        createdDateTime
        publishedDateTime
      }
    }
  }
}
        """



from typing import Any, Dict, Optional, List, TypedDict
from pydantic import BaseModel, field_validator


# Define typed dictionaries for nested structures
class SkillDict(TypedDict):
    name: str
    prettyName: str
    highlighted: bool


class AmountDict(TypedDict):
    rawValue: str
    currency: str
    displayValue: str


class LocationDict(TypedDict):
    city: Optional[str]
    country: Optional[str]
    timezone: Optional[str]
    state: Optional[str]
    offsetToUTC: Optional[str]


class ClientDict(TypedDict):
    edcUserId: Optional[str]
    totalHires: Optional[int]
    totalPostedJobs: Optional[int]
    location: Optional[LocationDict]
    totalSpent: Optional[Dict[str, float]]
    totalReviews: Optional[int]
    totalFeedback: Optional[float]
    companyRid: Optional[str]
    companyName: Optional[str]
    lastContractPlatform: Optional[str]
    lastContractRid: Optional[str]
    lastContractTitle: Optional[str]
    hasFinancialPrivacy: Optional[bool]


class JobSchema(BaseModel):
    id: str
    title: str
    description: str
    category: str
    total_applicants: int
    preferred_freelancer_location: Optional[str]
    preferred_freelancer_location_mandatory: bool
    premium: bool
    experience_level: str
    amount_raw_value: str
    amount_currency: str
    amount_display_value: str
    skills: List[SkillDict]
    hourly_budget_type: Optional[str]
    hourly_budget_min: Optional[float]
    hourly_budget_max: Optional[float]
    duration: Optional[str]
    client_not_sure_fields: Optional[str]
    client_private_fields: Optional[str]
    freelancer_client_relation: Optional[str]
    local_job_user_distance: Optional[str]
    weekly_budget_raw_value: Optional[str]
    weekly_budget_currency: Optional[str]
    weekly_budget_display_value: Optional[str]
    engagement_duration_id: Optional[str]
    engagement_duration_label: Optional[str]
    engagement_duration_weeks: Optional[int]
    client_edc_user_id: Optional[str]
    client_total_hires: Optional[int]
    client_total_posted_jobs: Optional[int]
    client_total_spent: Optional[float]
    client_location_city: Optional[str]
    client_location_country: Optional[str]
    client_location_timezone: Optional[str]
    client_location_state: Optional[str]
    client_location_offset_to_utc: Optional[str]
    client_total_reviews: Optional[int]
    client_total_feedback: Optional[float]
    client_company_rid: Optional[str]
    client_company_name: Optional[str]
    client_last_contract_platform: Optional[str]
    client_last_contract_rid: Optional[str]
    client_last_contract_title: Optional[str]
    client_has_financial_privacy: Optional[bool]
    occupations_category_id: Optional[str]
    occupations_category_pref_label: Optional[str]
    created_date_time: Optional[str]  # Consider using datetime
    published_date_time: Optional[str]  # Consider using datetime

    @classmethod
    def create_instance(cls, data: Dict[str, Any]) -> "JobSchema":
        # Safely get nested dictionary values
        amount: Dict[str, Any] = data.get("amount") or {}
        weekly_budget: Dict[str, Any] = data.get("weeklyBudget") or {}
        engagement_duration: Dict[str, Any] = data.get("engagementDuration") or {}
        client: ClientDict = data.get("client", {})
        client_location: Optional[LocationDict] = client.get("location")
        client_total_spent: Dict[str, Any] = client.get("totalSpent") or {}
        occupations_category: Dict[str, Any] = data.get("occupationsCategory", {}) or {}
        hourly_budget_min: Dict[str, Any] = data.get("hourlyBudgetMin") or {}
        hourly_budget_max: Dict[str, Any] = data.get("hourlyBudgetMax") or {}

        return cls(
            id=data.get("id", ""),  # singleLineText
            title=data.get("title", ""),  # singleLineText
            description=data.get("description", ""),
            category=data.get("category", ""),  # singleLineText
            total_applicants=data.get("totalApplicants", 0),  # number
            preferred_freelancer_location=data.get(
                "preferredFreelancerLocation"
            ),  # singleLineText
            preferred_freelancer_location_mandatory=data.get(
                "preferredFreelancerLocationMandatory", ""
            ),  # checkbox
            premium=data.get("premium", False),  # checkbox
            experience_level=data.get("experienceLevel", ""),  # singleLineText
            amount_raw_value=amount.get("rawValue", 0),  # number
            amount_currency=amount.get("currency", ""),  # singleLineText
            amount_display_value=amount.get("displayValue", ""),  # singleLineText
            skills=data.get("skills", []),  # multipleSelects
            hourly_budget_type=data.get("hourlyBudgetType"),  # singleLineText
            hourly_budget_min=hourly_budget_min.get("rawValue"),  # number
            hourly_budget_max=hourly_budget_max.get("rawValue"),  # number
            duration=data.get("duration"),  # singleLineText
            client_not_sure_fields=data.get("clientNotSureFields"),  # singleLineText
            client_private_fields=data.get("clientPrivateFields"),  # singleLineText
            freelancer_client_relation=data.get(
                "freelancerClientRelation"
            ),  # singleLineText
            local_job_user_distance=data.get("localJobUserDistance"),  # singleLineText
            weekly_budget_raw_value=weekly_budget.get("rawValue"),  # number
            weekly_budget_currency=weekly_budget.get("currency"),  # singleLineText
            weekly_budget_display_value=weekly_budget.get(
                "displayValue"
            ),  # singleLineText
            engagement_duration_id=engagement_duration.get("id"),  # singleLineText
            engagement_duration_label=engagement_duration.get(
                "label"
            ),  # singleLineText
            engagement_duration_weeks=engagement_duration.get("weeks"),  # number
            client_edc_user_id=client.get("edcUserId"),  # singleLineText
            client_total_hires=client.get("totalHires"),  # number
            client_total_posted_jobs=client.get("totalPostedJobs"),  # number
            client_total_spent=client_total_spent.get("rawValue"),  # number
            client_location_city=(
                client_location.get("city") if client_location else None
            ),  # singleLineText
            client_location_country=(
                client_location.get("country") if client_location else None
            ),  # singleLineText
            client_location_timezone=(
                client_location.get("timezone") if client_location else None
            ),  # singleLineText
            client_location_state=(
                client_location.get("state") if client_location else None
            ),  # singleLineText
            client_location_offset_to_utc=(
                client_location.get("offsetToUTC") if client_location else None
            ),  # singleLineText
            client_total_reviews=client.get("totalReviews"),  # number
            client_total_feedback=client.get("totalFeedback"),  # number
            client_company_rid=client.get("companyRid"),  # singleLineText
            client_company_name=client.get("companyName"),  # singleLineText
            client_last_contract_platform=client.get(
                "lastContractPlatform"
            ),  # singleLineText
            client_last_contract_rid=client.get("lastContractRid"),  # singleLineText
            client_last_contract_title=client.get(
                "lastContractTitle"
            ),  # singleLineText
            client_has_financial_privacy=client.get("hasFinancialPrivacy"),  # checkbox
            occupations_category_id=occupations_category.get("id"),  # singleLineText
            occupations_category_pref_label=occupations_category.get(
                "prefLabel"
            ),  # singleLineText
            created_date_time=data.get("createdDateTime"),  # singleLineText
            published_date_time=data.get("publishedDateTime"),  # singleLineText
        )

    @field_validator("preferred_freelancer_location", mode="before")
    @classmethod
    def preferred_freelancer_location_validator(
        cls, value: List[str] | None
    ) -> str | None:
        if isinstance(value, list):
            return " ".join(value)
        return value

    def __str__(self) -> str:
        return f"Job(id='{self.id}', title='{self.title}')"

    def __repr__(self) -> str:
        return self.__str__()

    class Meta:
        orm_mode = True



from rich.console import Console
from tortoise import Tortoise
from contextlib import asynccontextmanager

console = Console()


@asynccontextmanager
async def db_context(db_url: str):
    try:
        await Tortoise.init(
            db_url=db_url, modules={"models": ["upwork_fetcher.models"]}
        )
        await Tortoise.generate_schemas(safe=True)
        # console.print("Tortoise-ORM started")
        yield
    finally:
        await Tortoise.close_connections()


# Usage example:
# async with db_context():
#     # your database operations here



import asyncio
from typing import Any, Dict, List, Tuple
from urllib.parse import urlencode

import click
from gql.transport.requests import RequestsHTTPTransport
import gql
from rich.console import Console
from rich.table import Table

import requests

from upwork_fetcher.config import load_config, save_config
from upwork_fetcher.models.dbcontext import dbcontext
from upwork_fetcher.models.jobs import Job
from upwork_fetcher.queries.market_place_job import MarketplaceJobPostingsSearchQuery
from upwork_fetcher.schema import JobSchema

console = Console()


# CLIENT_ID = "13417fab9471a23c91ba14de24089ec3"  # Replace with your Upwork client ID
# CLIENT_SECRET = "db7c0b78592ad05b"  # Replace with your Upwork client secret
# REDIRECT_URI = "http://localhost:8000/upwork-callback"  # Replace with your redirect URI
# CODE = "ab09c682a28f7423b4e3f3018bb9b045"
# Upwork API URLs
AUTH_URL = "https://www.upwork.com/ab/account-security/oauth2/authorize"
TOKEN_URL = "https://www.upwork.com/api/v3/oauth2/token"


class UpworkService:

    def __init__(self, client_id: str, client_secret: str, redirect_uri: str):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.config: Dict[str, Any]

    def fetch(
        self,
        search_expressions: Tuple[str] | None,
        db_url: str,
        offset: int = 0,
        limit: int = 50,
        verbose: bool = False,
    ):
        with console.status("Refreshing Access Token..."):
            self.refresh_access_token()

        # search_expression = ", ".join(search_expressions or [])
        self.config = load_config()

        if search_expressions is None:
            search_expressions = tuple[str]()

        for search_expression in search_expressions:
            console.print(f"Searching on jobs about: '{search_expression}'")

            with console.status(f"searching on jobs..."):
                data = self.fetch_data(search_expression, offset, limit)

            if data is None:
                console.print(
                    f"[bold red]Failed to fetch jobs about: '{search_expression}'"
                )
                raise click.Abort()

            console.print(f"[bold green]{len(data)} jobs fetched successfully.")

            jobs = self.convert_dict_to_job_schema(data)

            if verbose:
                self._draw_table(jobs, table_title=f"Jobs about: '{search_expression}'")

            with console.status(f"Saving jobs into DB...") as status:
                data, new_data = asyncio.get_event_loop().run_until_complete(
                    self.split_data(jobs)
                )
                status.console.print(f"{len(new_data)} new jobs found")

                new_data = asyncio.get_event_loop().run_until_complete(
                    self.save_to_db(new_data)
                )
                status.console.print(f"{len(new_data)} jobs saved to DB")

    def _draw_table(self, jobs: List[JobSchema], table_title: str = "Fetched jobs"):
        table = Table(title=table_title, show_header=True, header_style="bold magenta")
        table.add_column("#", style="dim", justify="left")
        table.add_column("ID", style="dim", justify="left")
        table.add_column("Job Title", justify="left")

        for index, job in enumerate(jobs, start=1):
            table.add_row(str(index), job.id, job.title)

        console.print(table)

    async def split_data(self, data: List[JobSchema]):
        # config: Dict[str, Any] = load_config()
        async with dbcontext.db_context(db_url=self.config.get("db_url")):
            ids: List[str] = [job.id for job in data]
            existing_records: List[Dict[str, Any]] = await Job.filter(
                id__in=ids
            ).values("id")
            existing_ids: set[str] = {record["id"] for record in existing_records}

            # Filter out existing data
            new_data: List[JobSchema] = [
                job for job in data if job.id not in existing_ids
            ]

            # console.print(f"{len(new_data)} new jobs found")
            return data, new_data

    async def save_to_db(self, data: List[JobSchema]):
        # config = load_config()
        async with dbcontext.db_context(db_url=self.config.get("db_url")):

            list_of_jobs: List[Job] = []
            for job in data:
                list_of_jobs.append(Job(**job.model_dump()))

            await Job.bulk_create(list_of_jobs, ignore_conflicts=True)

        return data

    def get_authorization_url(self):
        """Returns the authorization URL for the Upwork API to get the code."""
        params = {
            "client_id": self.client_id,
            "response_type": "code",
            "redirect_uri": self.redirect_uri,
        }
        return f"{AUTH_URL}?{urlencode(params)}"

    def get_access_token(self, code: str):
        data = {
            "grant_type": "authorization_code",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "code": code,
            "redirect_uri": self.redirect_uri,
        }
        response = requests.post(
            TOKEN_URL,
            data=data,
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )

        response.raise_for_status()
        return response.json()

    # def get_refresh_token(self, auth_code: str):
    def _get_refresh_token_and_refresh_token(self, refresh_token: str):
        data = {
            "grant_type": "refresh_token",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "refresh_token": refresh_token,
        }
        response = requests.post(TOKEN_URL, data=data)
        response.raise_for_status()
        return response.json()

    def refresh_access_token(self):
        config = load_config()

        refresh_token = config["refresh_token"]

        new_tokens = self._get_refresh_token_and_refresh_token(refresh_token)

        self.access_token = new_tokens["access_token"]

        save_config(
            {
                "access_token": new_tokens["access_token"],
                "refresh_token": new_tokens["refresh_token"],
                "expires_in": new_tokens["expires_in"],
            }
        )
        # click.echo(load_config())

    def fetch_data(
        self,
        search_expression: str | None,
        offset: int = 0,
        limit: int = 50,
    ):

        # Define the transport with headers for authentication or other purposes
        transport = RequestsHTTPTransport(
            url="https://api.upwork.com/graphql",
            headers={
                "Authorization": f"Bearer {self.access_token}",
            },
        )

        # Create a GraphQL client
        client = gql.Client(transport=transport, fetch_schema_from_transport=True)

        # Define a query
        query = gql.gql(MarketplaceJobPostingsSearchQuery)
        # for search_expression in search_expressions:
        variable_values: Dict[str, Any] = {
            "marketPlaceJobFilter": {
                "searchExpression_eq": search_expression,
                "pagination_eq": {"after": str(offset), "first": limit},
            },
            "searchType": "JOBS_FEED",
            # "jobType_eq": "HOURLY", # FIXED
            "sortAttributes": [{"field": "RECENCY"}],
        }
        if search_expression:
            variable_values["marketPlaceJobFilter"][
                "searchExpression_eq"
            ] = search_expression
        # if skill_expression:
        #     variable_values["marketPlaceJobFilter"][
        #         "skillExpression_eq"
        #     ] = skill_expression
        # if title_expression:
        #     variable_values["marketPlaceJobFilter"][
        #         "titleExpression_eq"
        #     ] = title_expression

        try:
            result = client.execute(  # type: ignore
                document=query,
                variable_values=variable_values,
            )
            return result["marketplaceJobPostingsSearch"]["edges"]

        except Exception as e:
            click.echo(str(e))

    def convert_dict_to_job_schema(self, data: List[Dict[str, Any]]) -> List[JobSchema]:
        jobs: List[JobSchema] = []
        for edge in data:
            try:
                jobs.append(JobSchema.create_instance(edge["node"]))
            except Exception:
                continue

        return jobs



from tortoise import fields
from tortoise.models import Model


class UpworkConfig(Model):
    id = fields.IntField(pk=True)
    db_url = fields.TextField(null=True)
    access_token = fields.TextField(null=True)
    refresh_token = fields.TextField(null=True)
    token_type = fields.TextField(null=True)
    expires_in = fields.IntField(null=True)
    client_id = fields.TextField(null=True)
    client_secret = fields.TextField(null=True)
    redirect_uri = fields.TextField(null=True)
    code = fields.TextField(null=True)

    class Meta(Model.Meta):
        table = "upwork_configs"



import click

from upwork_fetcher.config import load_config
from upwork_fetcher.commands import fetch, setup


@click.group()
@click.version_option()
@click.pass_context
def cli(ctx: click.Context):
    config = load_config()
    ctx.obj = config


cli.add_command(fetch)
cli.add_command(setup)



from typing import Tuple
import click
from pydantic import PostgresDsn, ValidationError
from rich.console import Console

from upwork_fetcher.config import load_config, save_config, set_db_url
from upwork_fetcher.services.upwork_service import UpworkService


console = Console()


def validate_db_url(db_url: str):
    try:
        # Validate database URL using PostgresDsn
        db_url_validated = PostgresDsn(db_url)
    except ValidationError:
        console.print(f"[bold red]Invalid database URL format.")
        raise click.Abort()

    if not db_url_validated.path or len(db_url_validated.path) <= 1:
        console.print(f"[bold red]database must be provided")
        raise click.Abort()

    return db_url


@click.command()
@click.option(
    "--search-expression",
    "-s",
    multiple=True,
    help="Generic search filter supports partial Lucene syntax",
)
@click.option("--offset", prompt=False, default=0, help="The offset of the jobs")
@click.option("--limit", prompt=False, default=50, help="The limit of the jobs")
@click.option("--db-url", help="The database url")
@click.option("--verbose", "-v", is_flag=True, default=False, help="Verbose mode")
@click.pass_context
def fetch(
    ctx: click.Context,
    search_expression: Tuple[str],
    db_url: str | None,
    offset: int,
    limit: int,
    verbose: bool,
):
    """
    Fetch the jobs from the Upwork API
    """
    if not search_expression:
        console.print(
            "[bold red]You must specify at least one search expression. Run `upwork-fetcher --help`"
        )
        raise click.Abort()
    if db_url:
        validate_db_url(db_url)
        set_db_url(db_url)
        ctx.obj["db_url"] = db_url
        save_config({"db_url": db_url})
    elif ctx.obj.get("db_url"):
        set_db_url(ctx.obj["db_url"])
    else:
        database_url: str = click.prompt("Enter the database url")
        db_url = validate_db_url(database_url)
        set_db_url(db_url)
        save_config({"db_url": db_url})

    config = load_config()
    if (
        config.get("client_id") is None
        or config.get("client_secret") is None
        or config.get("redirect_uri") is None
        or config.get("code") is None
    ):
        console.print(
            "[bold red]You must setup the Upwork API Credentials first. Run `upwork-fetcher setup`"
        )
        raise click.Abort()

    upwork_service = UpworkService(
        client_id=config["client_id"],
        client_secret=config["client_secret"],
        redirect_uri=config["redirect_uri"],
    )

    upwork_service.fetch(
        search_expression,
        str(db_url),
        offset,
        limit,
        verbose,
    )


@click.command()
@click.option("--client_id", help="The client id")
@click.option("--client_secret", help="The client secret")
@click.option("--redirect_uri", help="The redirect uri")
@click.option("--db-url", help="The database url for storing credentials")
def setup(client_id: str, client_secret: str, redirect_uri: str, db_url: str | None = None):
    """
    Setup the Upwork API Credentials
    """
    
    # Set db_url if provided
    if db_url:
        validate_db_url(db_url)
        set_db_url(db_url)
        save_config({"db_url": db_url})
    else:
        # Check if db_url is already configured
        config = load_config()
        if config.get("db_url"):
            set_db_url(config["db_url"])
        else:
            # Prompt for db_url
            database_url: str = click.prompt("Enter the database url for storing credentials")
            db_url = validate_db_url(database_url)
            set_db_url(db_url)
            save_config({"db_url": db_url})

    upwork_service = UpworkService(
        client_id=client_id, client_secret=client_secret, redirect_uri=redirect_uri
    )
    url = upwork_service.get_authorization_url()

    save_config(
        {
            "client_id": client_id,
            "client_secret": client_secret,
            "redirect_uri": redirect_uri,
        }
    )

    console.print(
        f"\n\nGo to this url and copy the code from the query params: [underline blue]{url}\n\n"
    )
    code = click.prompt("Enter the code")

    tokens = upwork_service.get_access_token(code)

    save_config(tokens)
    save_config({"code": code})



import asyncio
import json
import os
from pathlib import Path
from typing import Any, Dict, Optional
from tortoise import Tortoise

from upwork_fetcher.models.dbcontext import dbcontext
from upwork_fetcher.models.upwork_config import UpworkConfig


# Fallback to file if database is not available
CONFIG_DIR = Path.home() / ".upwork_fetcher"
CONFIG_FILE = CONFIG_DIR / "config.json"

# Global variable to store db_url
_db_url: Optional[str] = None


def set_db_url(db_url: str):
    """Set the database URL for config storage"""
    global _db_url
    _db_url = db_url


def get_db_url() -> Optional[str]:
    """Get the database URL, checking environment variable first"""
    return os.getenv("UPWORK_DB_URL") or _db_url


async def _load_config_from_db() -> Dict[str, Any]:
    """Load configuration from database"""
    db_url = get_db_url()
    if not db_url:
        return {}

    async with dbcontext.db_context(db_url=db_url):
        # Get the latest config entry
        config_entry = await UpworkConfig.first()
        print(config_entry)
        if config_entry:
            config_dict: Dict[str, Any] = {
                "db_url": config_entry.db_url,
                "access_token": config_entry.access_token,
                "refresh_token": config_entry.refresh_token,
                "token_type": config_entry.token_type,
                "expires_in": config_entry.expires_in,
                "client_id": config_entry.client_id,
                "client_secret": config_entry.client_secret,
                "redirect_uri": config_entry.redirect_uri,
                "code": config_entry.code,
            }
            print(f"{config_dict = }")
            # Remove None values
            return {k: v for k, v in config_dict.items() if v is not None}

        return {}


async def _save_config_to_db(data: Dict[str, Any]):
    """Save configuration to database"""
    db_url = get_db_url()
    if not db_url:
        raise ValueError("Database URL not set")

    async with dbcontext.db_context(db_url=db_url):
        # Get existing config or create new one
        config_entry = await UpworkConfig.first()

        if config_entry:
            # Update existing entry
            for key, value in data.items():
                if hasattr(config_entry, key):
                    setattr(config_entry, key, value)
            await config_entry.save()
        else:
            # Create new entry
            config_data: Dict[str, Any] = {"db_url": db_url}
            config_data.update(data)
            # Filter out None values to avoid database constraints
            config_data = {k: v for k, v in config_data.items() if v is not None}
            await UpworkConfig.create(**config_data)


def load_config(config_path: Path = CONFIG_FILE) -> Dict[str, Any]:
    """Load configuration from database or file"""
    db_url = get_db_url()

    if db_url:
        # Try to load from database
        try:
            return asyncio.get_event_loop().run_until_complete(_load_config_from_db())
        except Exception as e:
            print(f"Warning: Failed to load config from database: {e}")
            # Fall back to file

    # Fallback to file-based config
    if not config_path.exists():
        CONFIG_DIR.mkdir(parents=True, exist_ok=True)

    if not config_path.exists():
        data = {}
        with open(config_path, "w") as file:
            json.dump(data, file, indent=4)

    with open(config_path, "r") as file:
        return json.load(file)


def save_config(data: Dict[str, Any]):
    """Save configuration to database or file"""
    db_url = get_db_url()
    print(f"{data = }")
    if db_url:
        # Save to database
        try:
            asyncio.get_event_loop().run_until_complete(_save_config_to_db(data))

            # Also update db_url if provided
            if "db_url" in data:
                set_db_url(data["db_url"])
            return
        except Exception as e:
            print(f"Warning: Failed to save config to database: {e}")
            # Fall back to file

    # Fallback to file-based config
    try:
        with open(CONFIG_FILE, "r") as file:
            config = json.loads(file.read())
    except (FileNotFoundError, json.JSONDecodeError):
        config = {}

    config.update(data)

    CONFIG_DIR.mkdir(parents=True, exist_ok=True)
    with open(CONFIG_FILE, "w") as file:
        json.dump(config, file, indent=4)



from tortoise.models import Model
from tortoise import fields


class Job(Model):
    id = fields.CharField(max_length=255, pk=True)
    title = fields.CharField(max_length=1024)
    description = fields.TextField()
    category = fields.CharField(max_length=1024)
    total_applicants = fields.IntField(null=True)
    preferred_freelancer_location = fields.CharField(max_length=1024, null=True)
    preferred_freelancer_location_mandatory = fields.BooleanField(null=True)
    premium = fields.BooleanField(null=True)
    experience_level = fields.CharField(max_length=50, null=True)
    amount_raw_value = fields.CharField(max_length=1024, null=True)
    amount_currency = fields.CharField(max_length=10, null=True)
    skills = fields.JSONField(null=True)  # type: ignore
    hourly_budget_type = fields.CharField(max_length=50, null=True)
    hourly_budget_min = fields.FloatField(null=True)
    hourly_budget_max = fields.FloatField(null=True)
    duration = fields.CharField(max_length=50, null=True)
    client_not_sure_fields = fields.TextField(null=True)
    client_private_fields = fields.TextField(null=True)
    freelancer_client_relation = fields.TextField(null=True)
    local_job_user_distance = fields.TextField(null=True)
    weekly_budget_raw_value = fields.CharField(max_length=1024, null=True)
    weekly_budget_currency = fields.CharField(max_length=10, null=True)
    weekly_budget_display_value = fields.CharField(max_length=1024, null=True)
    engagement_duration_id = fields.CharField(max_length=1024, null=True)
    engagement_duration_label = fields.CharField(max_length=1024, null=True)
    engagement_duration_weeks = fields.IntField(null=True)
    client_edc_user_id = fields.CharField(max_length=1024, null=True)
    client_total_hires = fields.IntField(null=True)
    client_total_posted_jobs = fields.IntField(null=True)
    client_total_spent = fields.FloatField(null=True)
    client_location_city = fields.CharField(max_length=1024, null=True)
    client_location_country = fields.CharField(max_length=100, null=True)
    client_location_timezone = fields.CharField(max_length=100, null=True)
    client_location_state = fields.CharField(max_length=100, null=True)
    client_location_offset_to_utc = fields.CharField(max_length=50, null=True)
    client_total_reviews = fields.IntField(null=True)
    client_total_feedback = fields.FloatField(null=True)
    client_company_rid = fields.CharField(max_length=1024, null=True)
    client_company_name = fields.CharField(max_length=1024, null=True)
    client_last_contract_platform = fields.CharField(max_length=1024, null=True)
    client_last_contract_rid = fields.CharField(max_length=1024, null=True)
    client_last_contract_title = fields.CharField(max_length=1024, null=True)
    client_has_financial_privacy = fields.BooleanField(null=True)
    occupations_category_id = fields.CharField(max_length=1024, null=True)
    occupations_category_pref_label = fields.CharField(max_length=1024, null=True)
    created_date_time = fields.DatetimeField()
    published_date_time = fields.DatetimeField()

    def __str__(self):
        return f"Job(id='{self.id}', title='{self.title}')"
